@@ -4,11 +4,19 @@ Pooled PostgreSQL database backend for Django.
 Requires psycopg 2: http://initd.org/projects/psycopg2
 """
 from django import get_version as get_django_version
-from django.db.backends.postgresql_psycopg2.base import CursorWrapper, \
+from django.db.backends.postgresql_psycopg2.base import \
     DatabaseWrapper as OriginalDatabaseWrapper
 from django.db.backends.signals import connection_created
 from threading import Lock
 import logging
+import sys
+
+try:
+    import psycopg2 as Database
+    import psycopg2.extensions
+except ImportError, e:
+    from django.core.exceptions import ImproperlyConfigured
+    raise ImproperlyConfigured("Error loading psycopg2 module: %s" % e)
 
 logger = logging.getLogger(__name__)
 
@@ -16,10 +24,34 @@ class PooledConnection():
     '''
     Thin wrapper around a psycopg2 connection to handle connection pooling.
     '''
-    def __init__(self, pool):
+    def __init__(self, pool, test_query=None):
         self._pool = pool
-        self._wrapped_connection = pool.getconn()
-        logger.debug("Checking out connection %s from pool %s" % (self._wrapped_connection, self._pool))
+        
+        # If passed a test query we'll run it to ensure the connection is available
+        if test_query:
+            self._wrapped_connection = None
+            num_attempts = 0
+            while self._wrapped_connection is None:
+                num_attempts += 1;
+                c = pool.getconn()
+                try:
+                    c.cursor().execute(test_query)
+                except Database.Error:
+                    pool.putconn(c, close=True)
+                    if num_attempts > self._pool.maxconn:
+                        logger.error("Unable to check out connection from pool %s" % self._pool)
+                        raise;
+                    else:
+                        logger.info("Closing dead connection from pool %s" % self._pool, 
+                                    exc_info=sys.exc_info())
+                else:
+                    if not c.autocommit:
+                        c.rollback()
+                    self._wrapped_connection = c
+        else:
+            self._wrapped_connection = pool.getconn()
+        
+        logger.debug("Checked out connection %s from pool %s" % (self._wrapped_connection, self._pool))
 
     def close(self):
         '''
@@ -43,6 +75,53 @@ uses our PooledDatabaseWrapper.)
 connection_pools = {}
 connection_pools_lock = Lock()
 
+pool_config_defaults = {
+    'MIN_CONNS': None, 
+    'MAX_CONNS': 1,
+    'TEST_ON_BORROW': False, 
+    'TEST_ON_BORROW_QUERY': 'SELECT 1'
+}
+
+def _set_up_pool_config(self):
+    '''
+    Helper to configure pool options during DatabaseWrapper initialization.
+    '''
+    self._max_conns = self.settings_dict['OPTIONS'].get('MAX_CONNS', pool_config_defaults['MAX_CONNS'])
+    self._min_conns = self.settings_dict['OPTIONS'].get('MIN_CONNS', self._max_conns)
+                
+    self._test_on_borrow = self.settings_dict["OPTIONS"].get('TEST_ON_BORROW', 
+                                                             pool_config_defaults['TEST_ON_BORROW'])
+    if self._test_on_borrow:
+        self._test_on_borrow_query = self.settings_dict["OPTIONS"].get('TEST_ON_BORROW_QUERY', 
+                                                                       pool_config_defaults['TEST_ON_BORROW_QUERY'])
+    else: 
+        self._test_on_borrow_query = None
+
+
+def _create_connection_pool(self, conn_params):
+    '''
+    Helper to initialize the connection pool.
+    '''
+    connection_pools_lock.acquire()
+    try:
+        # One more read to prevent a read/write race condition (We do this
+        # here to avoid the overhead of locking each time we get a connection.)
+        if (self.alias not in connection_pools or
+            connection_pools[self.alias]['settings'] != self.settings_dict):
+            logger.info("Creating connection pool for db alias %s" % self.alias)
+            logger.info("  using MIN_CONNS = %s, MAX_CONNS = %s, TEST_ON_BORROW = %s" % (self._min_conns,
+                                                                                         self._max_conns,
+                                                                                         self._test_on_borrow))
+
+            from psycopg2 import pool
+            connection_pools[self.alias] = {
+                'pool': pool.ThreadedConnectionPool(self._min_conns, self._max_conns, **conn_params),
+                'settings': dict(self.settings_dict),
+            }
+    finally:
+        connection_pools_lock.release()
+
+
 '''
 Simple Postgres pooled connection that uses psycopg2's built-in ThreadedConnectionPool
 implementation.  In Django, use this by specifying MAX_CONNS and (optionally) MIN_CONNS 
@@ -58,6 +137,10 @@ By default MIN_CONNS will be set to MAX_CONNS, which prevents connections from b
 If your load is spikey and you want to recycle connections, set MIN_CONNS to something lower 
 than MAX_CONNS.   I suggest it should be no lower than your 95th percentile concurrency for 
 your app server.
+
+If you wish to validate connections on each check out, specify TEST_ON_BORROW (set to True) 
+in the OPTIONS dictionary for the given db entry.  You can also provide an optional 
+TEST_ON_BORROW_QUERY, which is "SELECT 1" by default.
 '''
 class DatabaseWrapper16(OriginalDatabaseWrapper):
     '''
@@ -68,37 +151,39 @@ class DatabaseWrapper16(OriginalDatabaseWrapper):
       
     This code may be no longer needed!    
     '''
+    set_up_pool_config = _set_up_pool_config
+    create_connection_pool = _create_connection_pool
+    
+    def __init__(self, *args, **kwargs):
+        super(DatabaseWrapper16, self).__init__(*args, **kwargs)
+        self.set_up_pool_config()
+
     def get_new_connection(self, conn_params):
-        global connection_pools
-        
         # Is this the initial use of the global connection_pools dictionary for 
         # this python interpreter? Build a ThreadedConnectionPool instance and 
         # add it to the dictionary if so.
         if self.alias not in connection_pools or connection_pools[self.alias]['settings'] != self.settings_dict:
-            max_conns = conn_params.pop('MAX_CONNS', 1)
-            min_conns = conn_params.pop('MIN_CONNS', max_conns)
-        
-            connection_pools_lock.acquire()
-            try:
-                logger.debug("Creating connection pool for db alias %s" % self.alias)
+            for extra in pool_config_defaults.keys():
+                if extra in conn_params:
+                    del conn_params[extra]
 
-                from psycopg2 import pool
-                connection_pools[self.alias] = {
-                    'pool': pool.ThreadedConnectionPool(min_conns, max_conns, **conn_params),
-                    'settings': dict(self.settings_dict),
-                }
-            finally:
-                connection_pools_lock.release()
+            self.create_connection_pool(conn_params)
 
-        return PooledConnection(connection_pools[self.alias]['pool'])
+        return PooledConnection(connection_pools[self.alias]['pool'], test_query=self._test_on_borrow_query)
 
 
 class DatabaseWrapper14and15(OriginalDatabaseWrapper):
     '''
     For Django 1.4.x and 1.5.x
     '''
+    set_up_pool_config = _set_up_pool_config
+    create_connection_pool = _create_connection_pool
+    
+    def __init__(self, *args, **kwargs):
+        super(DatabaseWrapper14and15, self).__init__(*args, **kwargs)
+        self.set_up_pool_config()
+
     def _cursor(self):
-        global connection_pools
         settings_dict = self.settings_dict
         if self.connection is None or connection_pools[self.alias]['settings'] != settings_dict:
             # Is this the initial use of the global connection_pools dictionary for 
@@ -113,11 +198,10 @@ class DatabaseWrapper14and15(OriginalDatabaseWrapper):
                 conn_params = {
                     'database': settings_dict['NAME'],
                 }
-                max_conns = settings_dict['OPTIONS'].pop('MAX_CONNS', 1)
-                min_conns = settings_dict['OPTIONS'].pop('MIN_CONNS', max_conns)
                 conn_params.update(settings_dict['OPTIONS'])
-                if 'autocommit' in conn_params:
-                    del conn_params['autocommit']
+                for extra in ['autocommit'] + pool_config_defaults.keys():
+                    if extra in conn_params:
+                        del conn_params[extra]
                 if settings_dict['USER']:
                     conn_params['user'] = settings_dict['USER']
                 if settings_dict['PASSWORD']:
@@ -127,19 +211,10 @@ class DatabaseWrapper14and15(OriginalDatabaseWrapper):
                 if settings_dict['PORT']:
                     conn_params['port'] = settings_dict['PORT']
 
-                connection_pools_lock.acquire()
-                try:
-                    logger.debug("Creating connection pool for db alias %s" % self.alias)
+                self.create_connection_pool(conn_params)
 
-                    from psycopg2 import pool
-                    connection_pools[self.alias] = {
-                        'pool': pool.ThreadedConnectionPool(min_conns, max_conns, **conn_params),
-                        'settings': dict(settings_dict),
-                    }
-                finally:
-                    connection_pools_lock.release()
-
-            self.connection = PooledConnection(connection_pools[self.alias]['pool'])
+            self.connection = PooledConnection(connection_pools[self.alias]['pool'], 
+                                               test_query=self._test_on_borrow_query)
             self.connection.set_client_encoding('UTF8')
             tz = 'UTC' if settings.USE_TZ else settings_dict.get('TIME_ZONE')
             if tz:
@@ -169,12 +244,18 @@ class DatabaseWrapper13(OriginalDatabaseWrapper):
     '''
     For Django 1.3.x
     '''
+    set_up_pool_config = _set_up_pool_config
+    create_connection_pool = _create_connection_pool
+    
+    def __init__(self, *args, **kwargs):
+        super(DatabaseWrapper13, self).__init__(*args, **kwargs)
+        self.set_up_pool_config()
+
     def _cursor(self):
         '''
         Override _cursor to plug in our connection pool code.  We'll return a wrapped Connection
         which can handle returning itself to the pool when its .close() method is called.
         '''
-        global connection_pools
         from django.db.backends.postgresql.version import get_version
 
         new_connection = False
@@ -195,11 +276,10 @@ class DatabaseWrapper13(OriginalDatabaseWrapper):
                 conn_params = {
                     'database': settings_dict['NAME'],
                 }
-                max_conns = settings_dict['OPTIONS'].pop('MAX_CONNS', 1)
-                min_conns = settings_dict['OPTIONS'].pop('MIN_CONNS', max_conns)
                 conn_params.update(settings_dict['OPTIONS'])
-                if 'autocommit' in conn_params:
-                    del conn_params['autocommit']
+                for extra in ['autocommit'] + pool_config_defaults.keys():
+                    if extra in conn_params:
+                        del conn_params[extra]
                 if settings_dict['USER']:
                     conn_params['user'] = settings_dict['USER']
                 if settings_dict['PASSWORD']:
@@ -209,19 +289,10 @@ class DatabaseWrapper13(OriginalDatabaseWrapper):
                 if settings_dict['PORT']:
                     conn_params['port'] = settings_dict['PORT']
 
-                connection_pools_lock.acquire()
-                try:
-                    logger.debug("Creating connection pool for db alias %s" % self.alias)
+                self.create_connection_pool(conn_params)
 
-                    from psycopg2 import pool
-                    connection_pools[self.alias] = {
-                        'pool': pool.ThreadedConnectionPool(min_conns, max_conns, **conn_params),
-                        'settings': dict(settings_dict),
-                    }
-                finally:
-                    connection_pools_lock.release()
-
-            self.connection = PooledConnection(connection_pools[self.alias]['pool'])
+            self.connection = PooledConnection(connection_pools[self.alias]['pool'], 
+                                               test_query=self._test_on_borrow_query)
             self.connection.set_client_encoding('UTF8')
             self.connection.set_isolation_level(self.isolation_level)
             # We'll continue to emulate the old signal frequency in case any code depends upon it
@@ -255,12 +326,14 @@ version.  This is a bit hacky, what's a more elegant way?
 '''
 django_version = get_django_version()
 if django_version.startswith('1.3'):
+    from django.db.backends.postgresql_psycopg2.base import CursorWrapper
+    
     class DatabaseWrapper(DatabaseWrapper13):
         pass
 elif django_version.startswith('1.4') or django_version.startswith('1.5'):
     from django.conf import settings
-    from django.db.backends.postgresql_psycopg2.base import utc_tzinfo_factory
-    import psycopg2.extensions
+    from django.db.backends.postgresql_psycopg2.base import utc_tzinfo_factory, \
+        CursorWrapper
 
     # The force_str call around the password seems to be the only change from 
     # 1.4 to 1.5, so we'll use the same DatabaseWrapper class and make 
